from __future__ import annotations
import requests
import os
import logging
import shutil
from typing import Optional, Tuple

from packageurl import PackageURL

from ossbom.model.ossbom import OSSBOM


logger = logging.getLogger(__name__)


def can_report_to_github() -> bool:
    return not shutil.which("git") is None


def print_gh_action_errors(sbom: OSSBOM, package_path: str, post_to_github: bool = False) -> bool:
    """
    Print the errors in a format that can be consumed by GitHub Actions
    """
    details = None
    if post_to_github:
        if not can_report_to_github():
            logger.error("Git is not available, cannot post to GitHub")
        else:
            details = create_github_details()

    has_vulnerabilities = sbom.vulnerabilities is not None and len(sbom.vulnerabilities) > 0

    if has_vulnerabilities:
        for vuln in sbom.vulnerabilities:
            purl = PackageURL.from_string(vuln.purl)
            name = purl.name
            version = purl.version

            file, line = get_component_reference(name, package_path)
            message = f"WARNING: {name}:{version} contains malware. Remediate this immediately"
            print("Error: " + message)

            if post_to_github and details.is_pull_request:
                print(f"::error file={file},line={line}::{message}")
                append_to_github_output("error", message)
                post_comments_to_pull_request(details.token, details.repo, details.pull_number, details.commit_sha, message, file, line)
                post_comment_to_github_summary(details.token, details.repo, details.pull_number, message)
    else:
        print("No malware found")

    append_to_github_output(has_vulnerabilities, str(has_vulnerabilities).lower())

    return not has_vulnerabilities


def get_component_reference(component_name: str, package_path: str) -> Tuple[Optional[str], Optional[int]]:
    """
    Search through the files in the package until a requirements.txt or setup.py file
    that references the component name is found.

    Args:
        component (str): The name of the component to search for.
        package_path (str): The path to the package directory to search within.

    Returns:
        tuple: A tuple containing the file path and line number where the component is found.
               If the component is not found, returns (None, None).
    """
    files_to_check = ['requirements.txt', 'setup.py']

    for root, _, files in os.walk(package_path):
        for file in files:
            if file in files_to_check:
                file_path = os.path.join(root, file)
                try:
                    with open(file_path, 'r') as f:
                        for line_num, line in enumerate(f, start=1):
                            if component_name in line:
                                return file_path, line_num
                except Exception as e:
                    print(f"Error reading file {file_path}: {e}")

    # If component not found
    return None, None


class GitHubDetails:
    def __init__(self, token: Optional[str], repo: Optional[str], pull_number: str, commit_sha: Optional[str], is_pull_request: bool):
        self.token: Optional[str] = token
        self.repo: Optional[str] = repo
        self.pull_number: str = pull_number
        self.commit_sha: Optional[str] = commit_sha
        self.is_pull_request: bool = is_pull_request


def create_github_details() -> GitHubDetails:
    # Lazy import to avoid issues when running in context where GitHub API is not needed
    from github import Github

    token = os.getenv('GITHUB_TOKEN')
    repo = os.getenv('GITHUB_REPOSITORY')
    event_name = os.getenv('GITHUB_EVENT_NAME')
    pull_number = os.getenv('GITHUB_REF').split('/')[-2]
    is_pull_request = False
    commit_sha = None

    # Validate if this is a pull request or a push to a branch
    if event_name == 'pull_request':
        is_pull_request = True

        # If the pull number is a number, it is a pull request. Otherwise, it is a direct push
        if pull_number.isdigit():
            # Commit Sha
            g = Github(token)
            repo_object = g.get_repo(repo)
            pr = repo_object.get_pull(int(pull_number))
            commit_sha = pr.head.sha
            repo = repo_object.full_name

    return GitHubDetails(token, repo, pull_number, commit_sha, is_pull_request)


def post_comments_to_pull_request(token: str, repo: str, pull_number: str, commit_sha: str, comment: str, file_path: str, line: int = 0) -> None:

    data = {
        "body": comment,
        "commit_id": commit_sha,
        "path": file_path,
        "line": line,  # This is the line number in the file, not the diff position
        "side": "RIGHT"  # Comment on the right side (current version of the file)
    }

    # Send the POST request to GitHub API
    headers = {
        "Authorization": f"Bearer {token}",
        "Accept": "application/vnd.github.v3+json"
    }

    # API URL for pull request comments
    url = f"https://api.github.com/repos/{repo}/pulls/{pull_number}/comments"

    logger.debug(f"Building URL to comments POST: {url}")
    logger.debug(f"Using arguments: {data}")

    response = requests.post(url, headers=headers, json=data)

    if response.status_code == 201:
        # print("Comment added successfully.")
        pass
    else:
        print(f"Failed to add comment. Status code: {response.status_code}")
        print(response.json())  # Debugging information


def post_comment_to_github_summary(token: str, repo: str, pull_number: str, comment: str) -> None:
    # GitHub API URL to create a comment on the PR
    # Although we are posting on a Pull-Request we need to use the issues endpoint.
    # This allows us to post a comment that does not include a `commit_id` or `path`.
    url = f"https://api.github.com/repos/{repo}/issues/{pull_number}/comments"

    # Define the comment data
    data = {
        "body": comment
    }

    # Set up the headers with the GitHub token
    headers = {
        "Authorization": f"Bearer {token}",
        "Accept": "application/vnd.github.v3+json"
    }

    # Send the POST request to add the comment
    response = requests.post(url, json=data, headers=headers)

    # Check if the comment was successfully added
    if response.status_code == 201:
        print("Comment added successfully.")
    else:
        print(f"Failed to add comment. Status code: {response.status_code}")
        print(response.json())


def append_to_github_output(key: bool | str, value: str) -> None:
    if os.getenv("GITHUB_OUTPUT"):
        with open(os.getenv("GITHUB_OUTPUT"), "a", encoding="utf-8") as f:
            f.write(f"{key}={value}\n")



from __future__ import annotations

from pathlib import Path
from typing import Dict, Iterable, Tuple

from ossbom.model.component import Component
from ossbom.model.dependency_env import DependencyEnv
from ossbom.model.ossbom import OSSBOM
from ossprey.sbom_javascript import (
    get_all_node_modules_packages,
    node_modules_directory_exists,
)

Key = Tuple[str, str, str, str]  # (ptype, name, version, source)


def _iter_python_pkgs(root: Path) -> Iterable[tuple[str, str, Path]]:
    for p in root.rglob("*"):
        if p.is_dir() and (p.name.endswith(".dist-info") or p.name.endswith(".egg-info")):
            name, ver = None, None
            for meta in ("METADATA", "PKG-INFO"):
                f = p / meta
                if f.exists():
                    for line in f.read_text("utf-8", errors="ignore").splitlines():
                        if name is None and line.startswith("Name:"):
                            name = line.split(":", 1)[1].strip()
                        elif ver is None and line.startswith("Version:"):
                            ver = line.split(":", 1)[1].strip()
                        if name and ver:
                            break
            if name:
                yield name, ver or "", p


def _iter_node_modules(root: Path) -> Iterable[tuple[str, str, Path]]:
    seen: set[Path] = set()
    for nm in root.rglob("node_modules"):
        if any(parent in seen for parent in nm.parents):
            continue
        if node_modules_directory_exists(str(nm)):
            for c in get_all_node_modules_packages(str(nm)):
                yield c["name"], c.get("version", ""), nm
            seen.add(nm)


def update_sbom_from_docker(ossbom: OSSBOM, project_folder: str = "/") -> OSSBOM:
    root = Path(project_folder).resolve()

    # Aggregate locations per (type, name, version)
    buckets: Dict[Key, set[str]] = {}

    def add(ptype: str, name: str, version: str, loc: Path | str, source: str) -> None:
        key: Key = (ptype, name, version, source)
        buckets.setdefault(key, set()).add(str(loc))

    # Python
    for name, version, loc in _iter_python_pkgs(root):
        add("python", name, version, loc, "pkg_packages")

    # NPM
    for name, version, loc in _iter_node_modules(root):
        add("npm", name, version, loc, "node_modules")

    # Emit Components with all locations
    for (ptype, name, version, source), locs in buckets.items():
        ossbom.add_components(
            [
                Component.create(
                    name=name,
                    version=version,
                    type=ptype,
                    env=DependencyEnv.PROD,
                    source=source,
                    locations=sorted(locs),  # <-- list[str]
                )
            ]
        )

    return ossbom



from __future__ import annotations
import argparse
import os
from argparse import Namespace
from typing import Optional

from ossprey.modes import get_all_modes


# Used to pull booleans from env vars
def get_bool(value: Optional[str]) -> bool:
    return value.lower() in ("true", "1", "yes", "on") if value else False


###
# Parse the command line arguments
# Please Note: Everything argument needs a default value otherwise tests will fail
# @return: The parsed arguments
###
def parse_arguments() -> Namespace:

    parser = argparse.ArgumentParser(description="API URL:")
    parser.add_argument(
        "--url",
        type=str,
        help="The URL to process",
        default=os.getenv("INPUT_URL", "https://api.ossprey.com")
    )
    parser.add_argument(
        "--package", "--dir",
        dest="package",
        type=str,
        help="The package or directory to scan",
        default=os.getenv("INPUT_PACKAGE", os.getcwd())
    )
    parser.add_argument(
        "--dry-run",
        action="store_true",
        help="Dry run mode",
        default=get_bool(os.getenv("INPUT_DRY_RUN"))
    )
    parser.add_argument(
        "--github-comments",
        action="store_true",
        help="GitHub mode, will attempt to post comments to GitHub",
        default=get_bool(os.getenv("INPUT_GITHUB_COMMENTS"))
    )

    parser.add_argument(
        "--verbose", '-v',
        action="store_true",
        help="Verbose mode",
        default=get_bool(os.getenv("INPUT_VERBOSE"))
    )

    # Scanning methods
    parser.add_argument(
        '--mode',
        choices=get_all_modes(),
        help="Mode to generate the SBOM. Choose 'pipenv' to install the package or 'requirements' to provide a requirements file.",
        default=os.getenv("INPUT_MODE", 'auto')
    )

    # Authentication
    parser.add_argument(
        '--api-key',
        type=str,
        help="API Key to authenticate with the API.",
        default=os.getenv("API_KEY", None)
    )

    # Authentication
    parser.add_argument(
        '--soft-error',
        action="store_true",
        help="If the scan causes an error don't stop the CICD process from continuing",
        default=get_bool(os.getenv("INPUT_SOFT_ERROR"))
    )

    # Output
    parser.add_argument(
        '-o', '--output',
        type=str,
        help="Output file for the SBOM",
        default=os.getenv("INPUT_OUTPUT", None)
    )

    args = parser.parse_args()

    # Check if the API key is provided
    if args.api_key is None and args.dry_run is False:
        parser.error("--api_key or the environment variable API_KEY is required")

    return args



from __future__ import annotations

import json
import fnmatch
import os
from pathlib import Path
from typing import Iterable
from urllib.parse import urlparse

from ossbom.model.dependency_env import DependencyEnv
from ossbom.model.component import Component
from ossbom.model.ossbom import OSSBOM
from ossprey.sbom_javascript import (
    get_all_node_modules_packages,
    node_modules_directory_exists,
    get_all_package_lock_packages,
    get_all_yarn_lock_packages,
    resolve_github_duplicates,
)

# TODO All this code needs a refactor with the sbom_python and sbom_javascript code
IGNORE_PREFIXES = ("/proc", "/sys", "/dev", "/var/log", "/var/cache")


def _is_ignored(path: str) -> bool:
    p = os.path.normpath(path)
    for pref in IGNORE_PREFIXES:
        q = os.path.normpath(pref)
        if p == q or p.startswith(q + os.sep):
            return True
    return False


def iter_paths(
    root: Path, wildcard: str = "*", dir_only: bool = False
) -> Iterable[Path]:
    """Yield paths under root matching wildcard, pruning ignored dirs."""
    root = root if root.is_absolute() else root.resolve()
    if _is_ignored(str(root)):
        return

    for dirpath, dirnames, filenames in os.walk(root, topdown=True, followlinks=False):
        # prune before descending (skips /proc even when root is "/")
        dirnames[:] = [d for d in dirnames if not _is_ignored(os.path.join(dirpath, d))]
        base = Path(dirpath)

        if dir_only:
            for d in dirnames:
                if fnmatch.fnmatch(d, wildcard):
                    yield base / d
            continue

        for d in dirnames:
            if fnmatch.fnmatch(d, wildcard):
                yield base / d
        for f in filenames:
            if fnmatch.fnmatch(f, wildcard):
                yield base / f


def _iter_python_pkgs(root: Path) -> Iterable[Component]:

    python_pkgs = []
    for p in iter_paths(root, dir_only=True):
        if p.name.endswith(".dist-info") or p.name.endswith(".egg-info"):
            name, ver = None, None
            for meta in ("METADATA", "PKG-INFO"):
                f = p / meta
                if f.exists():
                    for line in f.read_text("utf-8", errors="ignore").splitlines():
                        if name is None and line.startswith("Name:"):
                            name = line.split(":", 1)[1].strip()
                        elif ver is None and line.startswith("Version:"):
                            ver = line.split(":", 1)[1].strip()
                        if name and ver:
                            break
            if name:
                python_pkgs.append((name, ver, p))

    components = []
    for name, version, loc in python_pkgs:
        direct_url = _get_direct_url(loc)
        ptype, cname, cver, source = _python_pkg_to_component_tuple(
            name, version, loc, direct_url
        )
        component = Component.create(
            type=ptype,
            name=cname,
            env=DependencyEnv.PROD.value,
            version=cver,
            source=source,
            location=[str(loc)],
        )
        components.append(component)
    return components


def _iter_node_modules(root: Path) -> Iterable[Component]:
    for nm in iter_paths(root, wildcard="node_modules", dir_only=True):
        path = nm.resolve().parent
        if node_modules_directory_exists(path):
            for c in get_all_node_modules_packages(path):
                c.add_location(str(path))
                yield c


def _iter_package_lock_files(root: Path) -> Iterable[Component]:
    for f in iter_paths(root, wildcard="package-lock.json"):
        for c in get_all_package_lock_packages(f.parent):
            c.add_location(str(f.parent))
            yield c


def _iter_yarn_lock_files(root: Path) -> Iterable[Component]:
    for f in iter_paths(root, wildcard="yarn.lock"):
        for c in get_all_yarn_lock_packages(f.parent):
            c.add_location(str(f.parent))
            yield c


def _get_direct_url(dist_info_dir: Path) -> dict | None:
    """Load direct_url.json from a .dist-info/.egg-info directory if present."""
    try:
        f = dist_info_dir / "direct_url.json"
        if f.exists():
            return json.loads(f.read_text("utf-8", errors="ignore"))
    except Exception:
        # Best-effort read; ignore corrupt files
        return None
    return None


def _github_repo_from_direct_url(direct_url: dict | None) -> str | None:
    """Return 'org/repo' if the direct_url points to GitHub; otherwise None."""
    if not direct_url:
        return None
    url = direct_url.get("url") or ""
    if not url:
        return None
    # Strip common VCS prefix
    if url.startswith("git+"):
        url = url[4:]
    parsed = urlparse(url)
    host = (parsed.netloc or "").lower()
    if not host.endswith("github.com"):
        return None
    parts = [p for p in parsed.path.split("/") if p]
    if len(parts) < 2:
        return None
    org, repo = parts[0], parts[1]
    if repo.endswith(".git"):
        repo = repo[:-4]
    return f"{org}/{repo}"


def _github_version_from_direct_url(direct_url: dict | None) -> tuple[str, str | None]:
    """Determine a (version, branch) from PEP 610 'vcs_info'."""
    vcs = (direct_url or {}).get("vcs_info") or {}
    requested = vcs.get("requested_revision")
    commit = vcs.get("commit_id")
    if commit:
        return commit[:12], requested
    elif requested:
        return "latest", requested
    else:
        return "latest", "main"


def _python_pkg_to_component_tuple(
    pypi_name: str, pypi_version: str, loc: Path, direct_url: dict | None
) -> tuple[str, str, str, str]:
    """
    Decide component identity for a Python package. Returns (ptype, name, version, source).
    If installed from GitHub (via direct_url.json), create a 'github' component using 'org/repo'.
    Else default to a 'pypi' component.
    """
    gh_repo = _github_repo_from_direct_url(direct_url)
    if gh_repo:
        ver, branch = _github_version_from_direct_url(direct_url)
        qs: list[str] = []
        if branch:
            qs.append(f"branch={branch}")
        qs.append(f"pypi_name={pypi_name}")
        qs.append(f"pypi_version={pypi_version}")
        return ("github", gh_repo, ver, "pkg_packages")

    return ("pypi", pypi_name, pypi_version, "pkg_packages")


def update_sbom_from_filesystem(ossbom: OSSBOM, project_folder: str = "/") -> OSSBOM:
    root = Path(project_folder).resolve()

    components = []
    # Python
    components.extend(_iter_python_pkgs(root))

    # NPM
    components.extend(_iter_node_modules(root))
    components.extend(_iter_package_lock_files(root))
    components.extend(_iter_yarn_lock_files(root))

    # Resolve potential NPM - Github duplications
    components = resolve_github_duplicates(components)

    # Add to SBOM
    ossbom.add_components(components)

    return ossbom



from __future__ import annotations
import logging
import subprocess
import json
import os
import sys
import shutil
from pathlib import Path
import tomllib
from packageurl import PackageURL

from ossbom.converters.factory import SBOMConverterFactory
from ossbom.model.ossbom import OSSBOM
from ossbom.model.component import Component

logger = logging.getLogger(__name__)


def get_cyclonedx_binary() -> str:
    if shutil.which("cyclonedx-py"):
        return "cyclonedx-py"

    venv_bin = Path(sys.executable).parent
    cmd = os.path.join(venv_bin, "cyclonedx-py")
    if os.path.exists(cmd):
        return cmd

    raise FileNotFoundError("cyclonedx-py binary not found.")


def create_sbom_from_requirements(requirements_file: str) -> OSSBOM:

    try:
        cmd = get_cyclonedx_binary()
        # This command generates an SBOM for the active virtual environment in JSON format
        result = subprocess.run(
            [cmd, 'requirements', requirements_file],
            check=True,
            capture_output=True,
            text=True,
            env=os.environ.copy()
        )

        ret = result.stdout

        cyclone_dict = json.loads(ret)

        ossbom = SBOMConverterFactory.from_cyclonedx_dict(cyclone_dict)

        return ossbom

    except subprocess.CalledProcessError as e:
        logger.error(f"Error running creating SBOM: {e}")
        logger.debug(e.stderr)
        logger.debug("--")
        logger.debug(e.stdout)
        raise e


def update_sbom_from_requirements(ossbom: OSSBOM, requirements_file: str) -> OSSBOM:
    sbom = create_sbom_from_requirements(requirements_file)
    ossbom.add_components(sbom.get_components())
    
    return ossbom


def create_sbom_from_env() -> OSSBOM:

    try:
        cmd = get_cyclonedx_binary()
        # This command generates an SBOM for the active virtual environment in JSON format
        result = subprocess.run(
            [cmd, 'environment'],
            check=True,
            capture_output=True,
            text=True,
            env=os.environ.copy()
        )

        ret = result.stdout

        cyclone_dict = json.loads(ret)

        ossbom = SBOMConverterFactory.from_cyclonedx_dict(cyclone_dict)

        return ossbom

    except subprocess.CalledProcessError as e:
        logger.error(f"Error running creating SBOM: {e}")
        logger.debug(e.stderr)
        logger.debug("--")
        logger.debug(e.stdout)
        raise e


def get_poetry_purls_from_lock(lockfile: str = "poetry.lock") -> list[PackageURL]:
    with open(lockfile, "rb") as f:
        lock_data = tomllib.load(f)

    purls = []
    for package in lock_data.get("package", []):
        name = package["name"]
        version = package["version"]
        purl = PackageURL(type="pypi", name=name.lower(), version=version)
        purls.append(purl)

    return purls


def update_sbom_from_poetry(ossbom: OSSBOM, package_dir: str) -> OSSBOM:

    if not os.path.exists(os.path.join(package_dir, "poetry.lock")):
        # Run poetry install to generate the poetry.lock file
        try:
            subprocess.run(['poetry', 'install'], cwd=package_dir, check=True)
        except subprocess.CalledProcessError as e:
            logger.error(f"Error running poetry install: {e}")
            raise e
        
    # Get the packages from the poetry.lock file
    purls = get_poetry_purls_from_lock(os.path.join(package_dir, "poetry.lock"))

    ossbom.add_components([Component.create(name=purl.name, version=purl.version, source="poetry", type="pypi") for purl in purls])

    return ossbom



from __future__ import annotations
import json
import logging
import sys

from ossprey.args import parse_arguments
from ossprey.github_actions_reporter import print_gh_action_errors
from ossprey.log import init_logging
from ossprey.scan import scan

logger = logging.getLogger(__name__)


def main() -> None:

    args = parse_arguments()

    init_logging(args.verbose)

    try:
        sbom = scan(
            args.package,
            mode=args.mode,
            local_scan=args.dry_run,
            url=args.url,
            api_key=args.api_key,
        )

        if sbom:

            if args.output:
                with open(args.output, "w") as f:
                    json.dump(sbom.to_dict(), f, indent=2)

            # Process the result
            ret = print_gh_action_errors(sbom, args.package, args.github_comments)

            if not ret:
                raise Exception("Error Malicious Package Found")

        sys.exit(0)

    except Exception as e:

        # Print the full stack trace
        if args.verbose:
            logger.exception(e)

        if args.soft_error:
            logger.error(f"Error: {e}")
            logger.error("Failing gracefully")
            sys.exit(0)
        else:
            logger.error(f"Error: {e}")
            sys.exit(1)






from __future__ import annotations
import os
import shutil
import subprocess
from typing import Optional

from ossbom.model.environment import Environment


def get_current_git_branch(path: str = ".") -> Optional[str]:

    if shutil.which("git") is None:
        return None  # git binary not available

    try:
        result = subprocess.run(
            ["git", "-C", path, "rev-parse", "--abbrev-ref", "HEAD"],
            stdout=subprocess.PIPE,
            stderr=subprocess.DEVNULL,
            check=True,
            text=True
        )
        return result.stdout.strip()
    except subprocess.CalledProcessError:
        return None  # not a git repo or other git error


def get_codespace_environment(package_name: str) -> Environment:
    github_org, github_repo = os.getenv("GITHUB_REPOSITORY").split("/")
    github_branch = get_current_git_branch()
    project = package_name
    machine_name = os.getenv("CODESPACE_NAME")
    product_env = "CODESPACE"
    return Environment.create(github_org, github_repo, github_branch, project, machine_name, product_env)


def get_gh_actions_environment(package_name: str) -> Environment:
    github_org, github_repo = os.getenv("GITHUB_REPOSITORY").split("/")
    github_branch = os.getenv("GITHUB_REF_NAME", None)
    project = package_name
    machine_name = os.getenv("CODESPACE_NAME")
    product_env = "GITHUB_ACTIONS"
    return Environment.create(github_org, github_repo, github_branch, project, machine_name, product_env)


def get_environment_details(package_name: str) -> Environment:

    if os.getenv("CODESPACES"):
        return get_codespace_environment(package_name)
    elif os.getenv("GITHUB_ACTIONS"):
        return get_gh_actions_environment(package_name)

    return Environment()



import pytest
import subprocess
import json
import os
import tempfile
import shutil

skip_in_gha = pytest.mark.skipif(
    os.getenv("GITHUB_ACTIONS") == "true",
    reason="Skipped on GitHub Actions.",
)


@pytest.fixture(autouse=True)
def cleanup():
    yield
    # subprocess.run(['docker', 'rm', '-f', 'simple_math_container'], check=False)


@pytest.mark.parametrize(
    "docker_folder, expected_packages, not_expected_packages, no_of_packages",
    [
        (
            "../test/docker_js_simple_math",
            {
                "npm": ["lodash", "axios", "@types/ms", "ms"],
                "github": ["ossprey/example_malicious_javascript"],
            },
            {
                "npm": ["mathlib", "send/ms"],
            },
            509,
        ),
        (
            "../test/docker_py_simple_math",
            {
                "pypi": ["fastapi", "uvicorn", "solana", "solders", "pydantic"],
                "github": ["ossprey/example_malicious_python"],
            },
            {
                "pypi": ["ossprey/example_malicious_python"],
            },
            27,
        ),
    ],
)
@skip_in_gha
def test_docker_build(
    docker_folder, expected_packages, not_expected_packages, no_of_packages
) -> None:
    if shutil.which("docker") is None:
        pytest.skip("Docker not available in environment")

    with tempfile.TemporaryDirectory() as tmpdir:
        # Run the build script from its directory
        script_dir = os.path.join(os.path.dirname(__file__), docker_folder)
        build_script = os.path.join(script_dir, "build.sh")
        subprocess.run(["bash", build_script, tmpdir], cwd=script_dir, check=True)
        sbom_path = os.path.join(tmpdir, "sbom.json")

        print("Attempt to get the sbom from the docker container")
        with open(sbom_path) as f:
            sbom = json.load(f)

    # Component count can vary slightly across platforms; ensure it's reasonable
    assert len(sbom["components"]) >= no_of_packages

    # Verify the GitHub-installed package is represented as a GitHub component
    for type, packages in expected_packages.items():
        for package in packages:
            print(f"Type: {type} Package: {package}")
            comp = next(
                (
                    c
                    for c in sbom["components"]
                    if c.get("type") == type and c.get("name") == package
                ),
                None,
            )
            assert comp is not None, f"Expected {type} {package} to be in SBOM"

    for type, packages in not_expected_packages.items():
        for package in packages:
            print(f"Type: {type} Package: {package}")
            comp = next(
                (
                    c
                    for c in sbom["components"]
                    if c.get("type") == type and c.get("name") == package
                ),
                None,
            )
            assert comp is None, f"Expected {type} {package} to NOT be in SBOM"

    # Basic structure checks
    assert isinstance(sbom.get("components"), list)
    assert all("name" in c and "type" in c for c in sbom["components"])



from __future__ import annotations
import json
import logging
import os
import re
import subprocess
from pathlib import Path

from typing import List

from ossbom.model.ossbom import OSSBOM
from ossbom.model.component import Component
from ossbom.model.dependency_env import DependencyEnv

logger = logging.getLogger(__name__)


class GitResolve:
    """
    Example github resolved value:
    "git+ssh://git@github.com/ossprey/example_malicious_javascript.git#cd3954ceeb60dd14abc065c909d9c9e1ce9d34e5",
    """

    def __init__(self, resolved):
        self.resolved = resolved
        if not self.resolved.startswith("git+"):
            raise ValueError("Invalid Git resolved value")

        self.url = self.resolved.split("+", 1)[1].split("#", 1)[0]
        self.version = self.resolved.split("#", 1)[1]
        self.name = self.url.split("/", 3)[-1].split(".git")[0]

    def get_type(self):
        return "github"

    def get_name(self):
        return self.name

    def get_version(self):
        return self.version


def resolve_github_duplicates(components: list[Component]) -> list[Component]:

    def hash(name, version, locations):
        ret = name + "-" + version + "-" + "-".join(locations)
        return ret

    # Get a list of npm names, versions and locations from GH components
    github_npm_details = set()
    github_components = [comp for comp in components if comp.type == "github"]
    for comp in github_components:
        metadata = comp.metadata or {}
        github_npm_details.add(
            hash(
                metadata.get("npm_name", ""),
                metadata.get("npm_version", ""),
                comp.location,
            )
        )

    logger.debug(github_npm_details)

    # Remove any NPM packages that match these details
    returned_components = []
    for comp in components:
        if comp.type == "npm":
            if hash(comp.name, comp.version, comp.location) not in github_npm_details:
                returned_components.append(comp)
            else:
                print(f"not appended: {comp.name}, {comp.version}")
        else:
            returned_components.append(comp)
    # returned_components = [
    #    comp
    #    for comp in components
    #    if comp.type == "npm"
    #    and hash(comp.name, comp.version, comp.location) not in github_npm_details
    # ]

    return returned_components


def exec_command(command: str, cwd: str | None = None) -> str:
    try:
        # Run the yarn command with the specified cwd
        result = subprocess.run(
            command.split(" "),
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            text=True,
            cwd=cwd if cwd else None,  # Explicitly set to None if not used
            check=True,
        )
        # Parse the output
        return result.stdout
    except subprocess.CalledProcessError as e:
        logger.error("Error while running yarn:", e)
        return e.stdout


def node_modules_directory_exists(project_folder: str | os.PathLike[str]) -> bool:
    return os.path.isdir(os.path.join(project_folder, "node_modules"))


def find_package_json_files(base_path: str | os.PathLike[str]) -> List[str]:
    package_files = []
    for root, dirs, files in os.walk(base_path):
        if "package.json" in files:
            package_files.append(Path(root) / "package.json")
    return package_files


def get_all_node_modules_packages(
    project_folder: str | os.PathLike[str],
) -> List[Component]:

    packages = []

    # Get all instances of the package.json file in the node_modules directory
    package_files = find_package_json_files(
        os.path.join(project_folder, "node_modules")
    )

    # Add all packages to the list
    for package_file in package_files:
        with open(package_file) as f:
            data = json.load(f)
            if "name" in data and "version" in data:
                name = data["name"]
                version = data["version"]

                # Make sure name is valid
                if name.startswith("<%="):
                    continue

                # Extract the name and version for each package
                packages.append({"name": name, "version": version})

    components = [
        Component.create(
            name=component["name"],
            version=component["version"],
            env=DependencyEnv.PROD.value,
            type="npm",
            source="node_modules",
        )
        for component in packages
    ]

    return components


def package_lock_file_exists(project_folder: str | os.PathLike[str]) -> bool:
    return os.path.isfile(os.path.join(project_folder, "package-lock.json"))


def get_all_package_lock_packages(
    project_folder: str | os.PathLike[str],
) -> List[Component]:
    # Get all packages in the package-lock.json file
    with open(os.path.join(project_folder, "package-lock.json")) as f:
        data = json.load(f)

    components = []
    for name, package in data["packages"].items():
        name = name.rsplit("node_modules/", 1)[-1]
        if name == "":
            continue
        version = package["version"]
        source = "package-lock.json"
        env = DependencyEnv.PROD.value
        is_github = package.get("resolved", "").find("github.com") != -1
        if is_github:
            # Create a github component
            type = "github"
            git_details = GitResolve(package["resolved"])
            metadata = {"npm_name": name, "npm_version": version}
            name = git_details.get_name()
            version = git_details.get_version()
            component = Component.create(
                name=name,
                version=version,
                env=env,
                type=type,
                source=source,
                metadata=metadata,
            )
        else:
            # Create an NPM package
            type = "npm"
            component = Component.create(
                name=name, version=version, env=env, type=type, source=source
            )
        components.append(component)

    return components


def package_json_file_exists(project_folder: str | os.PathLike[str]) -> bool:
    return os.path.isfile(os.path.join(project_folder, "package.json"))


def get_all_package_json_packages(project_folder: str | os.PathLike[str]) -> List[dict]:
    # Get all packages in the package.json file
    with open(os.path.join(project_folder, "package.json")) as f:
        data = json.load(f)

        deps = [
            {"name": dependency, "version": data["dependencies"][dependency]}
            for dependency in data["dependencies"]
        ]
        deps.extend(
            [
                {"name": dependency, "version": data["devDependencies"][dependency]}
                for dependency in data["devDependencies"]
            ]
        )

    return deps


def run_npm_dry_run(project_folder: str | os.PathLike[str]) -> str:
    return exec_command("npm install --dry-run --verbose", str(project_folder))


def get_all_npm_dry_run_packages(project_folder: str | os.PathLike[str]) -> List[dict]:
    # Get all packages from npm install --dry-run --verbose

    ret = []

    # Run the command
    result = run_npm_dry_run(project_folder)

    # Parse the output
    for line in result.split("\n"):
        # when the line starts with add
        if line.startswith("add"):
            # Extract the package name and version
            name = line.split(" ")[1]
            version = line.split(" ")[2]
            ret.append({"name": name, "version": version})

    # Extract the packages
    return ret


def yarn_lock_file_exists(project_folder: str | os.PathLike[str]) -> bool:
    return os.path.isfile(os.path.join(project_folder, "yarn.lock"))


def get_all_yarn_lock_packages(
    project_folder: str | os.PathLike[str],
) -> List[Component]:
    # Get all packages in the yarn.lock file
    file_path = os.path.join(project_folder, "yarn.lock")
    with open(file_path, "r") as f:
        content = f.read()

    # Regex to match package entries and their version
    package_regex = r'^(?:"|)([^\s"][^"]*)(?:"|):\n\s+version\s+"([^"]+)"'
    # Filter out all commented out lines first (starting with #)
    content = "\n".join(
        line for line in content.splitlines() if not line.strip().startswith("#")
    )
    # Find all matches in the yarn.lock content
    matches = re.finditer(package_regex, content, re.MULTILINE)

    package_data = []
    for match in matches:
        package_names = match.group(
            1
        )  # Full package spec (can include multiple package names)
        version = match.group(2)  # Version

        # Split package names if there are multiple (comma-separated)
        for name in package_names.split(", "):

            # Remove the last @ and everything after it
            name = name.rsplit("@", 1)[0].strip()

            # Validate the name is not an alias
            alias_match = re.match(r"([^@]+)@npm:([^@]+)", name)

            if alias_match:
                _, name = alias_match.groups()

            package_data.append({"name": name, "version": version.strip()})

    components = [
        Component.create(
            name=component["name"],
            version=component["version"],
            env=DependencyEnv.PROD.value,
            type="npm",
            source="yarn.lock",
        )
        for component in package_data
    ]

    return components


def run_yarn_install(project_folder: str | os.PathLike[str]) -> str:
    return exec_command(
        "yarn install --check-files -non-interactive", str(project_folder)
    )


def get_all_yarn_list_packages(
    project_folder: str | os.PathLike[str],
) -> List[Component]:
    # Get all packages from yarn list

    ret = []

    # Run the command
    result = exec_command("yarn list --json --no-progress", str(project_folder))

    list_json = result.strip().split("\n")[-1]

    # Parse the output
    data = json.loads(list_json)

    # Extract the packages
    for package in data["data"]["trees"]:
        name_and_version = package["name"].rsplit("@", 1)
        ret.append(
            Component.create(
                name=name_and_version[0],
                version=name_and_version[1],
                env=DependencyEnv.PROD.value,
                type="npm",
                source="yarn list",
            )
        )

    return ret


# Global functions
def update_sbom_from_npm(
    ossbom: OSSBOM, project_folder: str | os.PathLike[str]
) -> OSSBOM:

    # get all versions of a package in the node_modules directory
    if node_modules_directory_exists(project_folder):
        components = get_all_node_modules_packages(project_folder)
        ossbom.add_components(components)

    # get all packages in the package-lock.json file
    if package_lock_file_exists(project_folder):
        components = get_all_package_lock_packages(project_folder)
        ossbom.add_components(components)

    # npm install --dry-run --verbose - appears to be broken TODO fix
    # components = get_all_npm_dry_run_packages(project_folder)
    # ossbom.add_components([Component.create(name=component["name"], version=component["version"], env=DependencyEnv.PROD, type="npm", source="install") for component in components])

    return ossbom


def update_sbom_from_yarn(
    ossbom: OSSBOM, project_folder: str | os.PathLike[str], run_install: bool = False
) -> OSSBOM:

    if run_install:
        run_yarn_install(project_folder)

    # get all versions of a package in the node_modules directory
    if node_modules_directory_exists(project_folder):
        components = get_all_node_modules_packages(project_folder)
        ossbom.add_components(components)

    # get all packages in the package-lock.json file
    if yarn_lock_file_exists(project_folder):
        components = get_all_yarn_lock_packages(project_folder)
        ossbom.add_components(components)

    # get all packages in the package.json file
    # if package_json_file_exists(project_folder):
    #    packages.add_list(get_all_package_json_packages(project_folder), "package.json", DependencyEnv.PROD, type="npm")

    # yarn list
    components = get_all_yarn_list_packages(project_folder)
    ossbom.add_components(components)
    return ossbom



from __future__ import annotations

import pytest

from unittest.mock import patch, MagicMock

from ossprey.github_actions_reporter import print_gh_action_errors
from ossbom.model.ossbom import OSSBOM
import os
from ossprey.github_actions_reporter import GitHubDetails


def test_no_vulnerabilities():
    sbom = OSSBOM()
    sbom.vulnerabilities = []
    package_path = "test/path"

    with patch("builtins.print") as mock_print, patch("ossprey.github_actions_reporter.append_to_github_output") as mock_append:
        result = print_gh_action_errors(sbom, package_path)

        mock_print.assert_called_with("No malware found")
        mock_append.assert_called_once_with(False, "false")
        assert result is True

@pytest.mark.parametrize("append", [False, True])
def test_with_vulnerabilities(append):
    sbom = OSSBOM()
    sbom.vulnerabilities = [MagicMock(purl="pkg:pypi/testpkg@1.0.0")]
    package_path = "test/path"

    github_mock = GitHubDetails(
        is_pull_request=True,
        token=None,
        repo="repo",
        pull_number=1,
        commit_sha="sha"
    )
    with patch("ossprey.github_actions_reporter.create_github_details", return_value=github_mock), \
         patch("builtins.print") as mock_print, \
         patch("ossprey.github_actions_reporter.get_component_reference", return_value=("file.py", 10)), \
         patch("ossprey.github_actions_reporter.append_to_github_output") as mock_append:

        result = print_gh_action_errors(sbom, package_path, append)

        mock_print.assert_any_call("Error: WARNING: testpkg:1.0.0 contains malware. Remediate this immediately")

        if append:
            mock_print.assert_any_call("::error file=file.py,line=10::WARNING: testpkg:1.0.0 contains malware. Remediate this immediately")
            mock_append.assert_any_call(True, "true")
        else:
            mock_append.assert_called_once_with(True, "true")
        assert result is False


def test_with_github_posting():
    sbom = OSSBOM()
    sbom.vulnerabilities = [MagicMock(purl="pkg:pypi/testpkg@1.0.0")]
    package_path = "test/path"
    details_mock = MagicMock(is_pull_request=True, token="token", repo="repo", pull_number=1, commit_sha="sha")

    with patch("ossprey.github_actions_reporter.create_github_details", return_value=details_mock), \
         patch("ossprey.github_actions_reporter.get_component_reference", return_value=("file.py", 10)), \
         patch("ossprey.github_actions_reporter.append_to_github_output") as mock_append, \
         patch("ossprey.github_actions_reporter.post_comments_to_pull_request") as mock_post_comment, \
         patch("ossprey.github_actions_reporter.post_comment_to_github_summary") as mock_post_summary:

        result = print_gh_action_errors(sbom, package_path, post_to_github=True)

        mock_append.assert_called_with(True, "true")
        mock_post_comment.assert_called_once_with("token", "repo", 1, "sha", "WARNING: testpkg:1.0.0 contains malware. Remediate this immediately", "file.py", 10)
        mock_post_summary.assert_called_once_with("token", "repo", 1, "WARNING: testpkg:1.0.0 contains malware. Remediate this immediately")
        assert result is False



from __future__ import annotations
from ossprey.scan import scan
from ossbom.model.ossbom import OSSBOM
import pytest


@pytest.mark.parametrize("mode", ["python-requirements", "auto"])
def test_scan_py_success(mode: str) -> None:
    ret = scan("test/python_simple_math", mode=mode, local_scan=True)
    assert isinstance(ret, OSSBOM)
    assert [comp.name for comp in ret.get_components()] == ["numpy", "requests"]


def test_scan_py_success_pipenv() -> None:
    ret = scan("test/python_simple_math", mode="pipenv", local_scan=True)
    assert isinstance(ret, OSSBOM)

    result = [
        "certifi",
        "charset-normalizer",
        "idna",
        "numpy",
        "requests",
        "simple_math",
        "urllib3",
    ]
    assert [comp.name for comp in ret.get_components()] == result


@pytest.mark.parametrize("mode", ["auto", "poetry"])
def test_scan_poetry_success(mode: str) -> None:
    ret = scan("test/poetry_simple_math", mode=mode, local_scan=True)
    assert isinstance(ret, OSSBOM)

    result = ["certifi", "charset-normalizer", "idna", "numpy", "requests", "urllib3"]
    assert [comp.name for comp in ret.get_components()] == result


@pytest.mark.parametrize("mode", ["pipenv"])
def test_scan_poetry_success_pipenv(mode: str) -> None:
    ret = scan("test/poetry_simple_math", mode=mode, local_scan=True)
    assert isinstance(ret, OSSBOM)

    result = [
        "certifi",
        "charset-normalizer",
        "idna",
        "numpy",
        "poetry-simple-math",
        "requests",
        "urllib3",
    ]
    assert [comp.name for comp in ret.get_components()] == result


@pytest.mark.parametrize("mode", ["npm", "auto"])
def test_scan_npm_success(mode: str) -> None:
    ret = scan("test/npm_simple_math", mode=mode, local_scan=True)
    assert isinstance(ret, OSSBOM)
    assert len(ret.get_components()) == 322


@pytest.mark.parametrize(["mode", "num_components"], [("yarn", 323), ("auto", 323)])
def test_scan_yarn_success(mode: str, num_components: int) -> None:
    ret = scan("test/yarn_simple_math", mode=mode, local_scan=True)
    assert isinstance(ret, OSSBOM)
    assert len(ret.get_components()) == num_components


def test_scan_failure() -> None:
    with pytest.raises(Exception) as excinfo:
        scan(
            "test/python_simple_math_no_exist",
            mode="python-requirements",
            local_scan=True,
        )
    assert "Package test/python_simple_math_no_exist does not exist" in str(
        excinfo.value
    )


def test_scan_invalid_mode() -> None:
    with pytest.raises(Exception) as excinfo:
        scan("test/python_simple_math", mode="invalid-mode", local_scan=True)
    assert "Invalid scanning method" in str(excinfo.value)



from __future__ import annotations
import json
import logging
import os
import sys
import subprocess
import tempfile
import venv

logger = logging.getLogger(__name__)


class VirtualEnv:

    def __init__(self) -> None:
        self.original_sys_path = sys.path[:]
        self.temporary_dir = tempfile.TemporaryDirectory()
        self.temporary_files = []

        self.create_virtualenv()

    def __del__(self) -> None:
        self.exit()

        for file in self.temporary_files:
            os.remove(file)

        self.temporary_dir.cleanup()

    # Private methods
    def _exec(self, command: list[str], stdout=None):
        """
        Executes the specified command in the virtual environment.

        Parameters:
        command (list): The command to execute
        """
        try:
            if stdout:
                # Run the pip install command with the specified stdout
                result = subprocess.run(command, check=True, stdout=stdout, text=True)
            else:
                # Run the pip install command
                result = subprocess.run(
                    command, check=True, capture_output=True, text=True
                )
        except subprocess.CalledProcessError as e:
            logger.error(f"An error occurred while executing {' '.join(command)}':")
            logger.error(f"stdout:\n{e.stdout}")
            logger.error(f"stderr:\n{e.stderr}")
            # Throw the exception to the caller
            raise e

        return result

    def _get_pip_executable(self) -> str:
        """
        Returns the path to the pip executable in the virtual environment.
        """

        return (
            os.path.join(self.get_venv_dir(), "bin", "pip")
            if os.name != "nt"
            else os.path.join(self.temporary_dir, "Scripts", "pip.exe")
        )

    def _pip_install(self, package_path: str):
        """
        Installs the specified package in the virtual environment.

        Parameters:
        package_path (str): The path to the local package directory or .whl/.tar.gz file.
        """
        pip_executable = self._get_pip_executable()
        result = self._exec([pip_executable, "install", package_path])

        return result

    def _pip_list(self) -> list[dict]:
        """
        Lists all packages installed in the virtual environment.
        """
        pip_executable = self._get_pip_executable()
        result = self._exec([pip_executable, "list", "--format=json"])
        installed_packages = json.loads(result.stdout)
        return installed_packages

    # Public methods
    def get_venv_dir(self) -> str:
        return self.temporary_dir.name

    def create_virtualenv(self) -> None:
        """
        Creates a virtual environment in the specified directory.
        """
        venv_builder = venv.EnvBuilder(with_pip=True)
        venv_builder.create(self.get_venv_dir())
        logger.debug(f"Virtual environment created at {self.temporary_dir}")

    def install_package(self, package_path: str) -> None:
        """
        Installs the specified package in the virtual environment.

        Parameters:
        package_path (str): The path to the local package directory or .whl/.tar.gz file.
        """
        self._pip_install(package_path)
        logger.debug(
            f"Package '{package_path}' installed in virtual environment at {self.temporary_dir}"
        )

    def list_installed_packages(self) -> list[dict]:
        """
        Lists all packages installed in the virtual environment.
        """
        return self._pip_list()

    def create_requirements_file_from_env(self) -> str:

        requirements_file = tempfile.NamedTemporaryFile(suffix=".txt", delete=False)
        self.temporary_files.append(requirements_file.name)

        pip_executable = self._get_pip_executable()
        self._exec([pip_executable, "freeze"], stdout=requirements_file)

        return requirements_file.name

    def enter(self) -> None:
        """
        Replaces sys.path with paths from the specified virtual environment.
        """
        site_packages = os.path.join(
            self.get_venv_dir(),
            "lib",
            f"python{sys.version_info.major}.{sys.version_info.minor}",
            "site-packages",
        )
        stdlib = os.path.join(
            self.get_venv_dir(),
            "lib",
            f"python{sys.version_info.major}.{sys.version_info.minor}",
        )

        if not os.path.exists(site_packages):
            logger.error(
                f"Error: The virtual environment path '{site_packages}' does not exist."
            )
            raise FileNotFoundError(
                f"The virtual environment path '{site_packages}' does not exist."
            )

        # Save the original sys.path
        self.original_sys_path = sys.path[:]

        # Clear sys.path and set it to only include the virtualenv's site-packages and standard library
        sys.path[:] = [site_packages, stdlib]

    def exit(self) -> None:
        """
        Restores the original sys.path.
        """
        sys.path = self.original_sys_path



import json
import os
import pytest

from ossprey.main import main


def test_main_function(monkeypatch, capsys):
    monkeypatch.setattr("sys.argv", ["script.py"])
    monkeypatch.setenv("INPUT_PACKAGE", "test/python_simple_math")
    monkeypatch.setenv("INPUT_MODE", "python-requirements")
    monkeypatch.setenv("INPUT_DRY_RUN", "True")

    with pytest.raises(SystemExit) as excinfo:
        main()

    assert excinfo.value.code == 0

    captured = capsys.readouterr()
    print(captured.out)
    assert "No malware found" in captured.out


def test_main_function_with_output(monkeypatch, capsys):
    monkeypatch.setattr("sys.argv", ["script.py"])
    monkeypatch.setenv("INPUT_PACKAGE", "test/python_simple_math")
    monkeypatch.setenv("INPUT_MODE", "python-requirements")
    monkeypatch.setenv("INPUT_DRY_RUN", "True")
    monkeypatch.setenv("INPUT_OUTPUT", "sbom_output.json")

    with pytest.raises(SystemExit) as excinfo:
        main()

    captured = capsys.readouterr()
    print(captured.out)
    assert excinfo.value.code == 0

    assert "No malware found" in captured.out

    with open("sbom_output.json", "r") as f:
        sbom = json.load(f)
        assert sbom is not None
        # delete the file
        os.remove("sbom_output.json")


@pytest.mark.parametrize("soft_error, expected_ret", [
    ("True", 0),
    ("False", 1)])
def test_main_function_soft_error(monkeypatch, soft_error, expected_ret):
    monkeypatch.setattr("sys.argv", ["script.py"])
    monkeypatch.setenv("INPUT_PACKAGE", "test/python_simple_math_no_exist")
    monkeypatch.setenv("INPUT_MODE", "python-requirements")
    monkeypatch.setenv("INPUT_DRY_RUN", "True")
    monkeypatch.setenv("INPUT_SOFT_ERROR", soft_error)

    with pytest.raises(SystemExit) as excinfo:
        main()

    assert excinfo.value.code == expected_ret




import os
import logging

logger = logging.getLogger(__name__)


def get_all_modes() -> list[str]:
    return ['pipenv', 'python-requirements', 'poetry', 'npm', 'yarn', 'fs', 'auto']


def get_modes(directory: str) -> list[str]:
    """
    Get the modes from the directory.
    :param directory: The directory to scan.
    :return: A list of modes.
    """
    
    # get all files in the directory
    files = [f for f in os.listdir(directory) if os.path.isfile(os.path.join(directory, f))]
    logger.debug(f"Files in directory: {files}")
    modes = []

    # Check for requirements.txt
    if "requirements.txt" in files:
        modes.append("python-requirements")

    poetry_files = [
        "poetry.lock",
        "pyproject.toml"
    ]
    if any(poetry_file in files for poetry_file in poetry_files):
        # TODO handle poetry better in the future
        modes.append("poetry")

    npm_files = [
        "package-lock.json",
        "package.json",
        "node_modules"
    ]
    # Check for package.json
    if any(npm_file in files for npm_file in npm_files):
        modes.append("npm")

    # Check for yarn.lock
    if "yarn.lock" in files:
        modes.append("yarn")

    return modes



from __future__ import annotations
import sys
from pathlib import Path
import pytest
from unittest.mock import patch

from ossprey.sbom_python import create_sbom_from_env, create_sbom_from_requirements, get_cyclonedx_binary
from ossprey.virtualenv import VirtualEnv


def test_get_sbom():
    sbom = create_sbom_from_env()

    assert sbom.format == 'OSSBOM'


def test_get_sbom_from_venv():

    venv = VirtualEnv()
    venv.enter()

    # Install a package
    venv.install_package('numpy')

    requirements_file = venv.create_requirements_file_from_env()

    # Get the SBOM
    sbom = create_sbom_from_requirements(requirements_file)

    assert sbom.format == 'OSSBOM'
    assert len(sbom.components) == 1
    assert any(map(lambda x: x.name == 'numpy', sbom.components.values()))


def test_get_sbom_from_venv_local_package():

    venv = VirtualEnv()
    venv.enter()

    # Install a package
    venv.install_package('test/python_simple_math')
  
    requirements_file = venv.create_requirements_file_from_env()

    # Get the SBOM
    sbom = create_sbom_from_requirements(requirements_file)

    assert sbom.format == 'OSSBOM'
    assert len(sbom.components) == 7
    assert any(map(lambda x: x.name == 'simple_math', sbom.components.values()))


@patch("shutil.which")
def test_returns_shutil_which(mock_which):
    mock_which.return_value = "/usr/local/bin/cyclonedx-py"
    assert get_cyclonedx_binary() == "cyclonedx-py"


@patch("shutil.which", return_value=None)
def test_returns_venv_bin(mock_which):
    fake_bin = Path(sys.executable).parent / "cyclonedx-py"

    with patch("os.path.join", return_value=str(fake_bin)):
        result = get_cyclonedx_binary()
        assert str(fake_bin) in result


@patch("os.path.exists", return_value=False)
@patch("shutil.which", return_value=None)
def test_raises_when_not_found(mock_which, mock_exists):
    with pytest.raises(FileNotFoundError, match="cyclonedx-py binary not found."):
        get_cyclonedx_binary()



from __future__ import annotations
import json
import logging
import os
import requests
import time

from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry
from typing import Tuple, Final

logger = logging.getLogger(__name__)

# Global Cognito Constants
USER_POOL_DOMAIN = os.environ.get("USER_POOL_DOMAIN", "ossprey-auth")
CLIENT_ID = os.environ.get("CLIENT_ID", "77o2e7rf9ulhbpo78h6rdirqin")
REGION = os.environ.get("REGION", "eu-west-1")
TOKEN_ENDPOINT = f"https://{USER_POOL_DOMAIN}.auth.{REGION}.amazoncognito.com/oauth2/token"


class Ossprey:

    def __init__(self, api_url: str, api_key: str):
        self.api_url = api_url
        self.api_key = api_key

        if not self.api_key:
            raise Exception("API Key is null or empty")
        
        self.auth()
        self.session = self.create_session()

    def auth(self) -> None:
        """Authenticate with API Refresh Key and retrieve a temporary access token

        Raises:
            Exception: Failed to retrieve access token
            Exception: Failed to authenticate with API Key
        """
        logger.debug("Authenticating with API Key")
        data = {
            "grant_type": "refresh_token",
            "client_id": CLIENT_ID,
            "refresh_token": self.api_key
        }

        response = requests.post(TOKEN_ENDPOINT, data=data, headers={"Content-Type": "application/x-www-form-urlencoded"})
        if response.status_code != 200:
            logger.error("Failed to retrieve access token")
            logger.debug(f"Status code: {response.status_code}")
            logger.debug(f"Response: {response.text}")
            raise Exception(f"Authentication failed: {response.text}")

        access_token = response.json().get("access_token")
        if not access_token:
            logger.error("Access token is null or empty")
            raise Exception("Authentication failed: Access token is null or empty")

        self.access_token = access_token

        logger.debug("Authentication succeeded")

    # This takes a python dictionary and submits it to the API
    def validate(self, minibom: dict) -> dict | None:

        response = self.submit(minibom)

        match response.status_code:
            case 200:
                return response.json()
            case 202:
                json = response.json()
                sbom_id = json['sbom_id']
                scan_id = json['scan_id']
                return self.wait_for_completion(sbom_id, scan_id)
            case 429:
                logger.error("Rate limit exceeded")
                return None
            case _:
                logger.error("Failed to submit request")
                logger.debug(f"Status code: {response.status_code}")
                logger.debug(f"Response: {response.text}")

                data = response.json()
                if "message" in data:
                    logger.error(data["message"])
                return None

    def submit(self, json_bom: dict) -> requests.Response:

        # Get the url
        url = self.api_url + '/submit'

        logger.debug(f"JSON Submission: {json.dumps(json_bom)}")

        # Submit bom to API
        json_data = {"sbom": json_bom}
        headers = {'Content-Type': 'application/json', 'Authorization': f"Bearer {self.access_token}"}
        response = self.session.post(url, headers=headers, json=json_data)

        return response
        
    def wait_for_completion(self, sbom_id: str, scan_id: str) -> dict:
        url = self.api_url + '/status'

        params = {"sbom_id": sbom_id, "scan_id": scan_id}

        headers = {'Content-Type': 'application/json', 'Authorization': f"Bearer {self.access_token}"}
        for i in range(1, 20):
            time.sleep(i * i)

            response = self.session.get(url, headers=headers, params=params)
            if response.status_code not in [200, 202]:
                logger.error("Error returned when retrieving the results")
                logger.debug(f"Status code: {response.status_code}")
                logger.debug(f"Response: {response.text}")
                raise Exception("Error returned when retrieving the results")

            ret = response.json()
            if ret['status'] == 'SUCCEEDED':
                if 'output' in ret:
                    return ret['output']
                else:
                    raise Exception("Error no SBOM returned")

        logger.error("Scan took too long to complete")
        raise Exception("Scan took too long to complete")

    @staticmethod
    def create_session() -> requests.Session:
        """Return a Session that transparently retries on 503."""
        
        _RETRIES: Final = 5
        _BACKOFF: Final = 1.0  # seconds
        _STATUS_FORCELIST = (503,)
        _ALLOWED_METHODS: Final[Tuple[str, ...]] = (
            "GET",
            "POST",
        )

        retry = Retry(
            total=_RETRIES,
            backoff_factor=_BACKOFF,
            status_forcelist=_STATUS_FORCELIST,
            allowed_methods=_ALLOWED_METHODS,
            respect_retry_after_header=True,
        )
        adapter = HTTPAdapter(max_retries=retry)
        sess = requests.Session()
        sess.mount("http://", adapter)
        sess.mount("https://", adapter)
        return sess



import os
import pytest

from argparse import Namespace
from ossprey.args import parse_arguments


@pytest.fixture(autouse=True)
def set_ossprey_api_key(monkeypatch):
    monkeypatch.setenv("API_KEY", "SPECIAL_KEY")


@pytest.mark.parametrize(
    "cli_args,env_vars,expected",
    [
        # Test case 1: CLI only
        (
            ["script.py", "--url", "https://example.com", "--mode", "pipenv", "--verbose"],
            {},
            Namespace(
                url="https://example.com",
                package=os.getcwd(),
                dry_run=False,
                github_comments=False,
                verbose=True,
                mode="pipenv",
                api_key="SPECIAL_KEY",
                soft_error=False,
                output=None
            ),
        ),
        # Test case 2: Environment variable fallback
        (
            ["script.py", "--mode", "python-requirements"],
            {"INPUT_URL": "https://env-url.com", "INPUT_DRY_RUN": "true"},
            Namespace(
                url="https://env-url.com",
                package=os.getcwd(),
                dry_run=True,
                github_comments=False,
                verbose=False,
                mode="python-requirements",
                api_key="SPECIAL_KEY",
                soft_error=False,
                output=None
            ),
        ),
        # Test case 3: CLI overrides environment variables
        (
            ["script.py", "--url", "https://cli-url.com", "--dry-run", "--mode", "pipenv", "--api-key", "UNSPECIAL_KEY"],
            {"INPUT_URL": "https://env-url.com", "INPUT_DRY_RUN": "false"},
            Namespace(
                url="https://cli-url.com",
                package=os.getcwd(),
                dry_run=True,
                github_comments=False,
                verbose=False,
                mode="pipenv",
                api_key="UNSPECIAL_KEY",
                soft_error=False,
                output=None
            ),
        ),
        # Test case 4: Handles only env vars
        (
            ["script.py"],
            {"INPUT_URL": "https://env-url.com", "INPUT_PACKAGE": "newtest", "INPUT_MODE": "pipenv"},
            Namespace(
                url="https://env-url.com",
                package="newtest",
                dry_run=False,
                github_comments=False,
                verbose=False,
                mode="pipenv",
                api_key="SPECIAL_KEY",
                soft_error=False,
                output=None
            ),
        ),
        # Test case 5: Empty input
        (
            ["script.py"],
            {},
            Namespace(
                url="https://api.ossprey.com",
                package=os.getcwd(),
                dry_run=False,
                github_comments=False,
                verbose=False,
                mode="auto",
                api_key="SPECIAL_KEY",
                soft_error=False,
                output=None
            ),
        ),
    ],
)
def test_parse_arguments(monkeypatch, cli_args, env_vars, expected):
    # Mock sys.argv
    monkeypatch.setattr("sys.argv", cli_args)

    # Mock environment variables
    for key, value in env_vars.items():
        monkeypatch.setenv(key, value)

    # Call the parser
    args = parse_arguments()

    # Assert the parsed arguments match the expected result
    assert vars(args) == vars(expected)


def test_no_api_key_and_with_dryrun(monkeypatch):
    # Mock sys.argv with no mutually exclusive arguments
    monkeypatch.delenv("API_KEY")
    monkeypatch.setattr("sys.argv", ["script.py", "--mode", "pipenv", "--dry-run"])
    args = parse_arguments()
    print(args.api_key)

    expected = Namespace(
        url="https://api.ossprey.com",
        package=os.getcwd(),
        dry_run=True,
        github_comments=False,
        verbose=False,
        mode="pipenv",
        api_key=None,
        soft_error=False,
        output=None)
    
    assert vars(args) == vars(expected)


def test_no_api_key_and_no_dryrun(monkeypatch):
    # Mock sys.argv with no mutually exclusive arguments
    monkeypatch.delenv("API_KEY")

    monkeypatch.setattr("sys.argv", ["script.py", "--mode", "pipenv"])
    # Assert that the script raises a SystemExit error due to missing required arguments
    with pytest.raises(SystemExit) as excinfo:
        parse_arguments()

    # Validate the exit code and error message
    assert excinfo.value.code == 2  # argparse exits with code 2 for argument parsing errors


def test_mode_only_accepts_approved_values(monkeypatch):
    # Mock sys.argv with no mutually exclusive arguments
    monkeypatch.setattr("sys.argv", ["script.py", "--mode", "invalid"])

    # Assert that the script raises a SystemExit error due to missing required arguments
    with pytest.raises(SystemExit) as excinfo:
        parse_arguments()

    # Validate the exit code and error message
    assert excinfo.value.code == 2  # argparse exits with code 2 for argument parsing errors



from __future__ import annotations
import json
import logging
import os

from ossprey.environment import get_environment_details
from ossprey.log import init_logging
from ossprey.modes import get_modes, get_all_modes
from ossprey.sbom_python import update_sbom_from_requirements, update_sbom_from_poetry
from ossprey.sbom_javascript import update_sbom_from_npm, update_sbom_from_yarn
from ossprey.sbom_filesystem import update_sbom_from_filesystem
from ossprey.ossprey import Ossprey
from ossprey.virtualenv import VirtualEnv

from ossbom.converters.factory import SBOMConverterFactory
from ossbom.model.ossbom import OSSBOM

logger = logging.getLogger(__name__)


def scan(
    package_name: str,
    mode: str = "auto",
    local_scan: bool = False,
    url: str | None = None,
    api_key: str | None = None,
) -> OSSBOM:

    if mode == "auto":
        logger.debug("Auto mode selected")
        
        # Check the folder for files that map to different package managers
        modes = get_modes(package_name)
        if len(modes) == 0:
            logger.error("No package manager found")
            raise Exception("No package manager found in the directory")
    else:
        modes = [mode]

    logger.info(f"Scanning {package_name} with modes: {modes}")
    # If package location doesn't exist, raise an error
    if not os.path.exists(package_name):
        logger.error(f"Package {package_name} does not exist")
        raise Exception(f"Package {package_name} does not exist")

    sbom = OSSBOM()

    if any(mode not in get_all_modes() for mode in modes) or len(modes) == 0:
        raise Exception("Invalid scanning method: " + str(modes))

    if "pipenv" in modes:
        venv = VirtualEnv()
        venv.enter()

        venv.install_package(package_name)
        requirements_file = venv.create_requirements_file_from_env()

        sbom = update_sbom_from_requirements(sbom, requirements_file)

        venv.exit()

    if "python-requirements" in modes:
        sbom = update_sbom_from_requirements(sbom, package_name + "/requirements.txt")

    if "poetry" in modes:
        sbom = update_sbom_from_poetry(sbom, package_name)

    if "npm" in modes:
        sbom = update_sbom_from_npm(sbom, package_name)

    if "yarn" in modes:
        sbom = update_sbom_from_yarn(sbom, package_name)

    if "fs" in modes:
        sbom = update_sbom_from_filesystem(sbom, package_name)

    # Update sbom to contain the local environment
    env = get_environment_details(package_name)
    sbom.update_environment(env)

    logger.info(f"Scanning {len(sbom.get_components())}")

    if not local_scan:
        ossprey = Ossprey(url, api_key)

        # Compress to MINIBOM
        sbom = SBOMConverterFactory.to_minibom(sbom)

        sbom = ossprey.validate(sbom)
        if not sbom:
            raise Exception("Issue OSSPREY Service")

        # Convert to OSSBOM
        sbom = SBOMConverterFactory.from_minibom(sbom)

    logger.debug(json.dumps(sbom.to_dict(), indent=4))

    return sbom



from __future__ import annotations

from pathlib import Path
from typing import List

import json
import pytest

from ossbom.model.dependency_env import DependencyEnv
from ossbom.model.ossbom import OSSBOM
from ossprey import sbom_filesystem as fs
from ossbom.model.component import Component


def test_iter_python_pkgs_reads_metadata(tmp_path: Path) -> None:
    dist = tmp_path / "site-packages" / "foo-1.0.0.dist-info"
    dist.mkdir(parents=True)
    (dist / "METADATA").write_text("Name: foo\nVersion: 1.0.0\n")

    egg = tmp_path / "lib" / "bar.egg-info"
    egg.mkdir(parents=True)
    (egg / "PKG-INFO").write_text("Name: bar\nVersion: 2.0.0\n")

    results = list(fs._iter_python_pkgs(tmp_path))

    # Validate it returns components
    assert len(results) == 2
    assert all(isinstance(c, Component) for c in results)
    # Validate it includes the source and location
    pkg = next(c for c in results if c.name == "foo")
    print(pkg)
    assert pkg.version == "1.0.0"
    assert pkg.env == {DependencyEnv.PROD}
    assert pkg.type == "pypi"
    assert pkg.source == {"pkg_packages"}
    assert pkg.location == [str(dist)]

    assert all(
        val in ("pkg_packages", "egg_packages") for c in results for val in c.source
    )
    assert all(c.location == [str(dist)] or c.location == [str(egg)] for c in results)


def test_iter_python_pkgs_without_version(tmp_path: Path) -> None:
    dist = tmp_path / "pkg" / "nover.dist-info"
    dist.mkdir(parents=True)
    (dist / "METADATA").write_text("Name: nover\n")

    results = list(fs._iter_python_pkgs(tmp_path))

    # Expect a single Component with missing version (None)
    assert len(results) == 1
    assert isinstance(results[0], Component)
    assert results[0].name == "nover"
    assert results[0].version in (None, "")


def test_iter_ignored_dirs() -> None:
    restricted_path = Path("/proc")
    results = list(fs.iter_paths(restricted_path))

    assert results == []


def test_iter_node_modules_uses_helpers(
    tmp_path: Path, monkeypatch: pytest.MonkeyPatch
) -> None:
    nm1 = tmp_path / "a" / "node_modules"
    nm1.mkdir(parents=True)
    nm2 = tmp_path / "a" / "b" / "node_modules"
    nm2.mkdir(parents=True)

    def fake_exists(path: str | Path) -> bool:  # noqa: ARG001
        return True

    def fake_get_all(path: str | Path) -> List[Component]:
        # _iter_node_modules passes the parent directory of the node_modules folder
        p = Path(path)
        if p.name == "a":
            return [
                Component.create(
                    name="pkgA",
                    version="1.2.3",
                    env=DependencyEnv.PROD.value,
                    type="npm",
                    source="node_modules",
                )
            ]
        return [
            Component.create(
                name="@scope/pkgB",
                version="4.5.6",
                env=DependencyEnv.PROD.value,
                type="npm",
                source="node_modules",
            )
        ]

    monkeypatch.setattr(fs, "node_modules_directory_exists", fake_exists)
    monkeypatch.setattr(fs, "get_all_node_modules_packages", fake_get_all)

    results = list(fs._iter_node_modules(tmp_path))

    # Assert results only contains Components
    assert len(results) == 2
    assert all(isinstance(c, Component) for c in results)
    # Source is node_modules
    assert all(val == "node_modules" for c in results for val in c.source)
    # Location is present
    assert all(len(c.location) > 0 for c in results)

    # Validate at least one is valid
    print([c.location for c in results])
    print(str(nm1.parent))
    assert any(
        c.name == "pkgA" and c.version == "1.2.3" and c.location == [str(nm1.parent)]
        for c in results
    )


def test_update_sbom_from_filesystem_aggregates_locations(
    monkeypatch: pytest.MonkeyPatch, tmp_path: Path
) -> None:
    py_entries = [
        Component.create(
            name="requests",
            version="2.31.0",
            type="pypi",
            source="pkg_packages",
            env=DependencyEnv.PROD.value,
            location=[str(tmp_path / "pkg" / "requests-2.31.0.dist-info")],
        ),
        Component.create(
            name="requests",
            version="2.31.0",
            type="pypi",
            source="pkg_packages",
            env=DependencyEnv.PROD.value,
            location=[str(tmp_path / "pkg" / "requests-2.31.0.dist-info")],
        ),
    ]
    nm_entries = [
        Component.create(
            name="left-pad",
            version="1.3.0",
            env=DependencyEnv.PROD.value,
            type="npm",
            source="node_modules",
            location=[str(tmp_path / "a" / "b" / "node_modules" / "left-pad")],
        ),
    ]

    monkeypatch.setattr(fs, "_iter_python_pkgs", lambda root: iter(py_entries))
    monkeypatch.setattr(fs, "_iter_node_modules", lambda root: iter(nm_entries))

    sbom = OSSBOM()
    out = fs.update_sbom_from_filesystem(sbom, project_folder=str(tmp_path))

    assert out is sbom
    comps = list(sbom.components.values())
    names = {c.name for c in comps}
    assert names == {"requests", "left-pad"}

    # Validate basic metadata (locations no longer aggregated here)
    req = next(c for c in comps if c.name == "requests")
    assert req.version == "2.31.0"
    assert req.type == "pypi"

    # Validate locations
    print(req.location)
    assert req.location == [str(tmp_path / "pkg" / "requests-2.31.0.dist-info")] * 2
    left_pad = next(c for c in comps if c.name == "left-pad")
    assert left_pad.location == [
        str(tmp_path / "a" / "b" / "node_modules" / "left-pad")
    ]


def test_scan_handles_npm_and_pypi(
    monkeypatch: pytest.MonkeyPatch, tmp_path: Path
) -> None:
    py_entries = [
        Component.create(
            name="requests",
            version="2.31.0",
            type="pypi",
            source="pkg_packages",
            env=DependencyEnv.PROD.value,
        ),
        Component.create(
            name="requests",
            version="2.31.0",
            type="pypi",
            source="pkg_packages",
            env=DependencyEnv.PROD.value,
        ),
    ]
    nm_entries = [
        Component.create(
            name="left-pad",
            version="1.3.0",
            env=DependencyEnv.PROD.value,
            type="npm",
            source="node_modules",
        ),
    ]

    monkeypatch.setattr(fs, "_iter_python_pkgs", lambda root: iter(py_entries))
    monkeypatch.setattr(fs, "_iter_node_modules", lambda root: iter(nm_entries))

    sbom = OSSBOM()
    out = fs.update_sbom_from_filesystem(sbom, project_folder=str(tmp_path))

    assert out is sbom
    comps = list(sbom.components.values())
    names = {c.name for c in comps}
    assert names == {"requests", "left-pad"}

    # Find requests component and verify basic metadata
    req = next(c for c in comps if c.name == "requests")
    assert req.version == "2.31.0"
    assert req.type == "pypi"


def test_github_repo_from_direct_url_variants() -> None:
    # git+https URL with .git suffix
    d1 = {"url": "git+https://github.com/ossprey/example_malicious_python.git"}
    assert fs._github_repo_from_direct_url(d1) == "ossprey/example_malicious_python"

    # Plain https URL without .git
    d2 = {"url": "https://github.com/pallets/flask"}
    assert fs._github_repo_from_direct_url(d2) == "pallets/flask"

    # Non-GitHub URL should return None
    d3 = {"url": "https://gitlab.com/group/project.git"}
    assert fs._github_repo_from_direct_url(d3) is None


def test_github_version_from_direct_url_cases() -> None:
    # Prefer requested branch/tag name when present and different from commit
    d1 = {"vcs_info": {"requested_revision": "main", "commit_id": "abcdef0123456789"}}
    assert fs._github_version_from_direct_url(d1) == ("abcdef012345", "main")

    # Only commit present -> short commit, no branch
    d2 = {"vcs_info": {"commit_id": "0123456789abcdef0123"}}
    assert fs._github_version_from_direct_url(d2) == ("0123456789ab", None)

    # Neither present -> default latest
    d3 = {"vcs_info": {}}
    assert fs._github_version_from_direct_url(d3) == ("latest", "main")


def test_python_pkg_to_component_tuple_github_mapping(tmp_path: Path) -> None:
    loc = tmp_path / "pkg" / "example-0.1.0.dist-info"
    direct = {
        "url": "git+https://github.com/ossprey/example_malicious_python.git",
        "vcs_info": {"requested_revision": "main", "commit_id": "deadbeefcafebabe"},
    }
    ptype, name, ver, source = fs._python_pkg_to_component_tuple(
        "mathlib", "0.1.0", loc, direct
    )
    assert (ptype, name, ver, source) == (
        "github",
        "ossprey/example_malicious_python",
        "deadbeefcafe",
        "pkg_packages",
    )


def test_update_sbom_from_filesystem_emits_github_component(
    monkeypatch: pytest.MonkeyPatch, tmp_path: Path
) -> None:
    # Create a Python package installed metadata and simulate GitHub source via direct_url
    loc = tmp_path / "site-packages" / "example-0.1.0.dist-info"
    loc.mkdir(parents=True)
    (loc / "METADATA").write_text("Name: example_malicious_python\nVersion: 0.1.0\n")

    def fake_get_direct_url(path: Path) -> dict | None:  # noqa: ARG001
        return {
            "url": "https://github.com/ossprey/example_malicious_python",
            "vcs_info": {"requested_revision": "main"},
        }

    monkeypatch.setattr(fs, "_iter_node_modules", lambda root: iter(()))
    monkeypatch.setattr(fs, "_get_direct_url", fake_get_direct_url)

    sbom = OSSBOM()
    fs.update_sbom_from_filesystem(sbom, project_folder=str(tmp_path))

    comps = list(sbom.components.values())
    assert len(comps) == 1
    c = comps[0]
    assert c.type == "github"
    assert c.name == "ossprey/example_malicious_python"
    assert c.version == "latest"
    assert c.source == {"pkg_packages"}


def test__iter_python_pkgs_validity(tmp_path: Path) -> None:
    # Create a valid Python package metadata
    dist = tmp_path / "pkg" / "valid_pkg-1.0.0.dist-info"
    dist.mkdir(parents=True)
    (dist / "METADATA").write_text("Name: valid_pkg\nVersion: 1.0.0\n")

    # Create a second valid one with a github link
    dist2 = tmp_path / "pkg" / "valid_pkg_github-1.0.0.dist-info"
    dist2.mkdir(parents=True)
    (dist2 / "METADATA").write_text("Name: valid_pkg_github\nVersion: 1.0.0\n")
    (dist2 / "direct_url.json").write_text(
        '{"url": "git+https://github.com/ossprey/valid_pkg_github.git", "vcs_info": {"requested_revision": "main", "commit_id": "deadbeefcafebabe"}}'
    )

    results = list(fs._iter_python_pkgs(tmp_path))

    assert len(results) == 2
    assert all(isinstance(r, Component) for r in results)
    c = next(r for r in results if r.name == "valid_pkg")
    assert c.name == "valid_pkg"
    assert c.version == "1.0.0"
    assert c.type == "pypi"
    assert c.source == {"pkg_packages"}
    assert c.location == [str(dist)]

    d = next(r for r in results if r.name == "ossprey/valid_pkg_github")
    assert d.name == "ossprey/valid_pkg_github"
    assert d.version == "deadbeefcafebabe"[:12]
    assert d.type == "github"
    assert d.source == {"pkg_packages"}
    assert d.location == [str(dist2)]


def test__iter_node_modules(tmp_path):
    # Create a valid Node.js package metadata
    dist = tmp_path / "node_modules" / "valid_pkg"
    dist.mkdir(parents=True)
    (dist / "package.json").write_text('{"name": "valid_pkg", "version": "1.0.0"}')

    results = list(fs._iter_node_modules(tmp_path))

    assert len(results) == 1
    assert all(isinstance(r, Component) for r in results)
    c = results[0]
    assert c.name == "valid_pkg"
    assert c.version == "1.0.0"
    assert c.type == "npm"
    assert c.source == {"node_modules"}
    assert c.location == [str(tmp_path)]


def test__iter_package_lock_files(tmp_path):
    package_json = {
        "packages": {
            "node_modules/valid_pkg": {"version": "1.0.0"},
            "mathlib": {
                "version": "1.0.0",
                "resolved": "git+https://github.com/ossprey/special.git#abcdefgh",
            },
        }
    }

    # Create a valid package-lock.json
    lock_file = tmp_path / "package-lock.json"
    lock_file.write_text(json.dumps(package_json))

    results = list(fs._iter_package_lock_files(tmp_path))

    assert len(results) == 2
    assert all(isinstance(r, Component) for r in results)
    c = results[0]
    assert c.name == "valid_pkg"
    assert c.version == "1.0.0"
    assert c.type == "npm"
    assert c.source == {"package-lock.json"}
    assert c.location == [str(tmp_path)]

    d = results[1]
    assert d.name == "ossprey/special"
    assert d.version == "abcdefgh"
    assert d.type == "github"
    assert d.source == {"package-lock.json"}
    assert d.location == [str(tmp_path)]


# TODO BROKEN
def test__iter_yarn_lock_files(tmp_path):
    yarn_lock = """# THIS IS AN AUTOGENERATED FILE. DO NOT EDIT THIS FILE DIRECTLY.
# yarn lockfile v1


"@ampproject/remapping@^2.2.0":
  version "2.3.0"
  resolved "https://registry.npmjs.org/@ampproject/remapping/-/remapping-2.3.0.tgz"
  integrity sha512-30iZtAPgz+LTIYoeivqYo853f02jBYSd5uGnGpkFV0M3xOt9aN73erkgYAmZU43x4VfqcnLxW9Kpg3R5LC4YYw==
  dependencies:
    "@jridgewell/gen-mapping" "^0.3.5"
    "@jridgewell/trace-mapping" "^0.3.24"

"@babel/compat-data@^7.26.5":
  version "7.26.8"
  resolved "https://registry.npmjs.org/@babel/compat-data/-/compat-data-7.26.8.tgz"
  integrity sha512-oH5UPLMWR3L2wEFLnFJ1TZXqHufiTKAiLfqw5zkhS4dKXLJ10yVztfil/twG8EDTA4F/tvVNw9nOl4ZMslB8rQ==

    """

    # Create a valid yarn.lock
    lock_file = tmp_path / "yarn.lock"
    lock_file.write_text(yarn_lock)
    # tmp_path = Path("./test/docker_js_simple_math/javascript/")
    results = list(fs._iter_yarn_lock_files(tmp_path))

    assert len(results) == 2
    assert all(isinstance(r, Component) for r in results)
    c = results[0]
    assert c.name == "@ampproject/remapping"
    assert c.version == "2.3.0"
    assert c.type == "npm"
    assert c.source == {"yarn.lock"}
    assert c.location == [str(tmp_path)]

    d = results[1]
    assert d.name == "@babel/compat-data"
    assert d.version == "7.26.8"
    assert d.type == "npm"
    assert d.source == {"yarn.lock"}
    assert d.location == [str(tmp_path)]



from __future__ import annotations
import pytest
from unittest.mock import patch, mock_open
from typing import Any
from pathlib import Path
from ossprey.sbom_javascript import (
    GitResolve,
    exec_command,
    node_modules_directory_exists,
    find_package_json_files,
    get_all_node_modules_packages,
    package_lock_file_exists,
    get_all_package_lock_packages,
    package_json_file_exists,
    get_all_package_json_packages,
    run_npm_dry_run,
    get_all_npm_dry_run_packages,
    yarn_lock_file_exists,
    get_all_yarn_lock_packages,
    run_yarn_install,
    get_all_yarn_list_packages,
    update_sbom_from_npm,
    update_sbom_from_yarn,
)

from ossbom.model.ossbom import OSSBOM
from ossbom.model.component import Component


def test_exec_command() -> None:
    with patch("subprocess.run") as mock_run:
        mock_run.return_value.stdout = "output"
        assert exec_command("echo test") == "output"


def test_node_modules_directory_exists(tmp_path: Path) -> None:
    (tmp_path / "node_modules").mkdir()
    assert node_modules_directory_exists(tmp_path) is True


def test_find_package_json_files(tmp_path: Path) -> None:
    (tmp_path / "node_modules").mkdir()
    (tmp_path / "node_modules" / "package.json").write_text("{}")
    assert find_package_json_files(tmp_path) == [
        tmp_path / "node_modules" / "package.json"
    ]


def test_get_all_node_modules_packages(tmp_path: Path) -> None:
    (tmp_path / "node_modules").mkdir()
    (tmp_path / "node_modules" / "package.json").write_text(
        '{"name": "testpkg", "version": "1.0.0"}'
    )
    comps = get_all_node_modules_packages(tmp_path)
    assert len(comps) == 1
    c = comps[0]
    assert isinstance(c, Component)
    assert c.name == "testpkg"
    assert c.version == "1.0.0"


def test_package_lock_file_exists(tmp_path: Path) -> None:
    (tmp_path / "package-lock.json").write_text("{}")
    assert package_lock_file_exists(tmp_path) is True


def test_get_all_package_lock_packages(tmp_path: Path) -> None:
    (tmp_path / "package-lock.json").write_text(
        '{"packages": {"node_modules/testpkg": {"version": "1.0.0"}}}'
    )
    comps = get_all_package_lock_packages(tmp_path)
    assert len(comps) == 1
    c = comps[0]
    assert isinstance(c, Component)
    assert c.name == "testpkg"
    assert c.version == "1.0.0"
    assert c.source == {"package-lock.json"}


def test_get_all_package_lock_packages_ignore_blanks(tmp_path: Path) -> None:
    (tmp_path / "package-lock.json").write_text(
        '{"packages": {"": {}, "node_modules/testpkg": {"version": "1.0.0"}}}'
    )
    comps = get_all_package_lock_packages(tmp_path)
    assert len(comps) == 1
    c = comps[0]
    assert isinstance(c, Component)
    assert c.name == "testpkg"
    assert c.version == "1.0.0"
    assert c.source == {"package-lock.json"}


def test_package_json_file_exists(tmp_path: Path) -> None:
    (tmp_path / "package.json").write_text("{}")
    assert package_json_file_exists(tmp_path) is True


def test_get_all_package_json_packages(tmp_path: Path) -> None:
    (tmp_path / "package.json").write_text(
        '{"dependencies": {"testpkg": "1.0.0"}, "devDependencies": {"testpkg-dev": "1.0.0"}}'
    )
    assert get_all_package_json_packages(tmp_path) == [
        {"name": "testpkg", "version": "1.0.0"},
        {"name": "testpkg-dev", "version": "1.0.0"},
    ]


def test_run_npm_dry_run() -> None:
    with patch("ossprey.sbom_javascript.exec_command") as mock_exec_command:
        mock_exec_command.return_value = "add testpkg 1.0.0\n"
        assert get_all_npm_dry_run_packages(".") == [
            {"name": "testpkg", "version": "1.0.0"}
        ]


def test_yarn_lock_file_exists(tmp_path: Path) -> None:
    (tmp_path / "yarn.lock").write_text("")
    assert yarn_lock_file_exists(tmp_path) is True


def test_get_all_yarn_lock_packages(tmp_path: Path) -> None:
    (tmp_path / "yarn.lock").write_text('"testpkg@^1.0.0":\n  version "1.0.0"\n')
    comps = get_all_yarn_lock_packages(tmp_path)
    assert len(comps) == 1
    c = comps[0]
    assert isinstance(c, Component)
    assert c.name == "testpkg"
    assert c.version == "1.0.0"


def test_run_yarn_install() -> None:
    with patch("ossprey.sbom_javascript.exec_command") as mock_exec_command:
        mock_exec_command.return_value = "output"
        assert run_yarn_install(".") == "output"


def test_get_all_yarn_list_packages() -> None:
    with patch("ossprey.sbom_javascript.exec_command") as mock_exec_command:
        mock_exec_command.return_value = (
            '{"data": {"trees": [{"name": "testpkg@1.0.0"}]}}'
        )
        comps = get_all_yarn_list_packages(".")
    assert len(comps) == 1
    c = comps[0]
    assert isinstance(c, Component)
    assert c.name == "testpkg"
    assert c.version == "1.0.0"


# def test_update_sbom_from_npm() -> None:
#        mock_get_all_npm_dry_run_packages.return_value = [{"name": "testpkg", "version": "1.0.0"}]
#    with patch("ossprey.sbom_javascript.get_all_npm_dry_run_packages") as mock_get_all_npm_dry_run_packages:
#        sbom = OSSBOM()
#        sbom = update_sbom_from_npm(sbom, ".")
#        assert len(sbom.components) == 1
#
#        # Get only entry in sbom.components and confirm it's name value is testpkg
#        for component in sbom.components.values():
#            assert component.name == "testpkg"


def test_update_sbom_from_yarn() -> None:
    with patch(
        "ossprey.sbom_javascript.get_all_yarn_list_packages"
    ) as mock_get_all_yarn_list_packages:
        mock_get_all_yarn_list_packages.return_value = [
            Component.create(
                name="testpkg",
                version="1.0.0",
                env=None,
                type="npm",
                source="yarn list",
            )
        ]
        sbom = OSSBOM()
        sbom = update_sbom_from_yarn(sbom, ".")
        assert len(sbom.components) == 1

        # Get only entry in sbom.components and confirm it's name value is testpkg
        for component in sbom.components.values():
            assert component.name == "testpkg"


# GitResolve tests
def test_GitResolve_parses_https_url() -> None:
    gr = GitResolve("git+https://github.com/pallets/flask.git#abcdef0123")
    assert gr.get_type() == "github"
    assert gr.get_version() == "abcdef0123"
    assert gr.url == "https://github.com/pallets/flask.git"
    assert gr.get_name() == "pallets/flask"


def test_GitResolve_parses_ssh_url() -> None:
    gr = GitResolve(
        "git+ssh://git@github.com/ossprey/example_malicious_javascript.git#deadbeef"
    )
    assert gr.get_type() == "github"
    assert gr.get_version() == "deadbeef"
    assert gr.url == "ssh://git@github.com/ossprey/example_malicious_javascript.git"
    assert gr.get_name() == "ossprey/example_malicious_javascript"


def test_GitResolve_raises_without_git_prefix() -> None:
    with pytest.raises(ValueError):
        GitResolve("https://github.com/org/repo.git#123")


def test_GitResolve_without_dot_git_suffix() -> None:
    gr = GitResolve("git+https://github.com/org/repo#main")
    assert gr.get_type() == "github"
    assert gr.url == "https://github.com/org/repo"
    assert gr.get_name() == "org/repo"
    assert gr.get_version() == "main"



from __future__ import annotations
import os
import sys

from ossprey.virtualenv import VirtualEnv


def test_virtual_env_creation() -> None:

    venv = VirtualEnv()

    # Check if the virtual environment files exist
    assert os.path.isdir(os.path.join(venv.get_venv_dir(), 'bin'))


def test_accessing_virtual_env() -> None:

    venv = VirtualEnv()
    venv.enter()

    assert len([name.startswith(f"{venv.get_venv_dir()}/lib") for name in sys.path]) > 0

    installed_packages = venv.list_installed_packages()

    # Make sure we are in a new venv
    assert len(installed_packages) == 1
    assert installed_packages[0]['name'] == 'pip'


def test_package_installation() -> None:
    venv = VirtualEnv()
    venv.enter()

    # Install a package
    venv.install_package('numpy')

    installed_packages = venv.list_installed_packages()

    # Make sure we are in a new venv
    assert len(installed_packages) == 2
    assert any(map(lambda x: x['name'] == 'numpy', installed_packages))


def test_local_requirements_file() -> None:
    venv = VirtualEnv()
    venv.enter()
    venv.install_package('numpy')

    requirements_file = venv.create_requirements_file_from_env()

    # Read the file
    with open(requirements_file, 'r') as f:
        lines = f.readlines()
        assert len(lines) == 1
        assert lines[0].startswith('numpy')


def test_local_installation() -> None:
    venv = VirtualEnv()
    venv.enter()

    # Install a package
    venv.install_package('test/python_simple_math')

    installed_packages = venv.list_installed_packages()

    # Make sure we are in a new venv
    assert len(installed_packages) == 8
    assert any(map(lambda x: x['name'] == 'numpy', installed_packages))



from __future__ import annotations
import logging
import logging.config
from typing import Dict, Any


def get_logging_config(log_level: int) -> Dict[str, Any]:
    LOGGING_CONFIG = {
        'version': 1,
        'disable_existing_loggers': False,
        'formatters': {
            'standard': {
                'format': '%(asctime)s [%(levelname)s] %(name)s: %(message)s'
            },
        },
        'handlers': {
            'console': {
                'class': 'logging.StreamHandler',
                'level': 'DEBUG',
                'formatter': 'standard',
                'stream': 'ext://sys.stdout',
            },
            'file': {
                'class': 'logging.FileHandler',
                'level': 'INFO',
                'formatter': 'standard',
                'filename': 'app.log',
                'mode': 'a',
            },
        },
        'loggers': {
            '': {  # root logger
                'handlers': ['console', 'file'],
                'level': log_level,
                'propagate': True
            }
        }
    }

    return LOGGING_CONFIG


def init_logging(verbose: bool = False) -> None:
    log_level = logging.DEBUG if verbose else logging.WARNING

    config = get_logging_config(log_level)

    logging.config.dictConfig(config)
